@@ -10,15 +10,20 @@ import subprocess, sys
 import getpass
 import socket
 import platform
+import os.path
 
 def get_svn_props():
-    props = {}
+    props = {
+                "REVISION" : "unknown",
+                "LOCAL_MODIFICATIONS" : "unknown"
+            }
     #find revision:
     proc = subprocess.Popen("svnversion -n", stdin=None, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
     (out, _) = proc.communicate()
     if not out:
         print("could not get version information")
         return  props
+    out = out.decode('utf-8')
     pos = out.find(":")
     if pos>=0:
         out = out[pos+1:]
@@ -29,7 +34,7 @@ def get_svn_props():
     if not rev_str:
         print("could not parse version information from string: %s" % rev_str)
         return  props
-        
+
     rev = int(rev_str)
     props["REVISION"] = rev
     #find number of local files modified:
@@ -39,8 +44,8 @@ def get_svn_props():
     if not out:
         print("could not get status of local files")
         return  props
-    
-    for line in out.splitlines():
+
+    for line in out.decode('utf-8').splitlines():
         if sys.platform.startswith("win") and line.find("\\wcw"):
             """ windows is easily confused, symlinks for example - ignore them """
             continue
@@ -50,9 +55,11 @@ def get_svn_props():
     props["LOCAL_MODIFICATIONS"] = changes
     return props
 
-def append_properties_to_file(props):
-    #append to build_info.py:
-    f = open("./xpra/build_info.py", 'a')
+def save_properties_to_file(props):
+    filename = "./xpra/build_info.py"
+    if os.path.exists(filename):
+        os.unlink(filename)
+    f = open(filename, mode='w')
     for name,value in props.items():
         f.write("%s='%s'\n" % (name,value))
     f.close()
@@ -67,11 +74,7 @@ def main():
             }
     for k,v in get_svn_props().items():
         props[k] = v
-    append_properties_to_file(props)
-
-def append_svn_props():
-    props = get_svn_props()
-    append_properties_to_file(props)
+    save_properties_to_file(props)
 
 if __name__ == "__main__":
     main()
