@@ -9,17 +9,17 @@
 
 import sys
 import os.path
-import pygtk
-pygtk.require("2.0")
-import gtk
-import gobject
+from wimpiggy.gobject_compat import import_gtk, import_gdk, import_gobject, is_gtk3
+gtk = import_gtk()
+gdk = import_gdk()
+gobject = import_gobject()
 import webbrowser
 import time
 import datetime
 
 from xpra.platform import XPRA_LOCAL_SERVERS_SUPPORTED
 from xpra.scripts.main import ENCODINGS
-from xpra.keys import XMODMAP_MOD_DEFAULTS, XMODMAP_MOD_ADD, XMODMAP_MOD_CLEAR, get_gtk_keymap
+from xpra.keys import get_gtk_keymap
 from wimpiggy.util import gtk_main_quit_really
 from wimpiggy.log import Logger
 log = Logger()
@@ -79,6 +79,23 @@ def set_checkeditems(submenu, is_match_func):
             if a!=v:
                 x.set_active(v)
 
+def set_tooltip_text(widget, text):
+    if hasattr(gtk, "pygtk_version") and gtk.pygtk_version<(2,12):
+        #not available!
+        return
+    widget.set_tooltip_text(text)
+
+def CheckMenuItem(label):
+    """ adds a get_label() method for older versions of gtk which do not have it
+        beware that this label is not mutable!
+    """
+    cmi = gtk.CheckMenuItem(label)
+    if not hasattr(cmi, "get_label"):
+        def get_label():
+            return  label
+        cmi.get_label = get_label
+    return cmi
+
 class ClientExtrasBase(object):
 
     def __init__(self, client, opts):
@@ -95,7 +112,10 @@ class ClientExtrasBase(object):
             window_icon = self.get_icon_filename("xpra.png")
         if window_icon and os.path.exists(window_icon):
             try:
-                gtk.window_set_default_icon_from_file(window_icon)
+                if is_gtk3():
+                    gtk.Window.set_default_icon_from_file(window_icon)
+                else:
+                    gtk.window_set_default_icon_from_file(window_icon)
                 log.debug("set default window icon to %s", window_icon)
             except Exception, e:
                 log.error("failed to set window icon %s: %s, continuing", window_icon, e)
@@ -137,15 +157,14 @@ class ClientExtrasBase(object):
     def can_notify(self):
         return  False
 
-    def show_notify(self, dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout):
+    def show_notify(self, dbus_id, nid, app_name, replaces_id, app_icon, summary, body, expire_timeout):
         pass
 
-    def close_notify(self, id):
+    def close_notify(self, nid):
         pass
 
     def system_bell(self, window, device, percent, pitch, duration, bell_class, bell_id, bell_name):
-        import gtk.gdk
-        gtk.gdk.beep()
+        gdk.beep()
 
     def get_layout_spec(self):
         """ layout, variant, variants"""
@@ -158,12 +177,11 @@ class ClientExtrasBase(object):
         return  get_gtk_keymap()
 
     def get_keymap_modifiers(self):
-        return  XMODMAP_MOD_CLEAR, XMODMAP_MOD_ADD, {}, [], []
+        return  {}, [], []
 
     def get_keymap_spec(self):
-        """ xkbmap_print, xkbmap_query, xmodmap """
-        # default map for non xposix clients:
-        return None,None,XMODMAP_MOD_DEFAULTS
+        """ xkbmap_print, xkbmap_query """
+        return None,None
 
     def get_keyboard_repeat(self):
         """ (delay_ms,interval_ms) or None"""
@@ -174,12 +192,16 @@ class ClientExtrasBase(object):
             self.about_dialog.present()
             return
         dialog = gtk.AboutDialog()
-        def on_website_hook(dialog, web, *args):
-            webbrowser.open("http://xpra.org/")
-        def on_email_hook(dialog, mail, *args):
-            webbrowser.open("mailto://"+mail)
-        gtk.about_dialog_set_url_hook(on_website_hook)
-        gtk.about_dialog_set_email_hook(on_email_hook)
+        if not is_gtk3():
+            def on_website_hook(dialog, web, *args):
+                webbrowser.open("http://xpra.org/")
+            def on_email_hook(dialog, mail, *args):
+                webbrowser.open("mailto://"+mail)
+            gtk.about_dialog_set_url_hook(on_website_hook)
+            gtk.about_dialog_set_email_hook(on_email_hook)
+            xpra_icon = self.get_pixbuf("xpra.png")
+            if xpra_icon:
+                dialog.set_icon(xpra_icon)
         dialog.set_name("Xpra")
         from xpra import __version__
         dialog.set_version(__version__)
@@ -228,14 +250,15 @@ class ClientExtrasBase(object):
         window.set_destroy_with_parent(True)
         window.set_resizable(True)
         window.set_decorated(True)
-        pixbuf = None
-        if self.tray_icon:
-            pixbuf = self.get_pixbuf(self.tray_icon)
+        pixbuf = self.get_pixbuf("statistics.png")
         if not pixbuf:
-            pixbuf = self.get_pixbuf("xpra.png")
+            pixbuf = self.get_pixbuf(self.tray_icon)
         if pixbuf:
             window.set_icon(pixbuf)
-        window.set_position(gtk.WIN_POS_CENTER)
+        if is_gtk3():
+            window.set_position(gtk.WindowPosition.CENTER)
+        else:
+            window.set_position(gtk.WIN_POS_CENTER)
 
         # Contents box
         vbox = gtk.VBox(False, 0)
@@ -255,19 +278,26 @@ class ClientExtrasBase(object):
 
         # now add some rows with info:
         row = 0
+        if is_gtk3():
+            row = add_row(row, gtk.Label("PyGobject version"), gtk.Label(gobject._version))
+            row = add_row(row, gtk.Label("GTK version"), gtk.Label(gtk._version))
+            row = add_row(row, gtk.Label("GDK version"), gtk.Label(gdk._version))
+        else:
+            row = add_row(row, gtk.Label("PyGTK version"), gtk.Label(".".join([str(x) for x in gtk.pygtk_version])))
+            row = add_row(row, gtk.Label("GTK version"), gtk.Label(".".join([str(x) for x in gtk.gtk_version])))
+
         self.server_version_label = gtk.Label()
         row = add_row(row, gtk.Label("Server Version"), self.server_version_label)
         if self.client.server_platform:
             row = add_row(row, gtk.Label("Server Platform"), gtk.Label(self.client.server_platform))
         self.server_randr_label = gtk.Label()
         row = add_row(row, gtk.Label("Server RandR Support"), self.server_randr_label)
-        if self.client.can_ping:
-            self.server_load_label = gtk.Label()
-            row = add_row(row, gtk.Label("Server Load"), self.server_load_label)
-            self.server_latency_label = gtk.Label()
-            row = add_row(row, gtk.Label("Server Latency"), self.server_latency_label)
-            self.client_latency_label = gtk.Label()
-            row = add_row(row, gtk.Label("Client Latency"), self.client_latency_label)
+        self.server_load_label = gtk.Label()
+        row = add_row(row, gtk.Label("Server Load"), self.server_load_label)
+        self.server_latency_label = gtk.Label()
+        row = add_row(row, gtk.Label("Server Latency"), self.server_latency_label)
+        self.client_latency_label = gtk.Label()
+        row = add_row(row, gtk.Label("Client Latency"), self.client_latency_label)
         self.session_started_label = gtk.Label()
         row = add_row(row, gtk.Label("Session Started"), self.session_started_label)
         self.session_connected_label = gtk.Label()
@@ -286,7 +316,7 @@ class ClientExtrasBase(object):
                 return False
             self.client.send_ping()
             def settimedeltastr(label, from_time):
-                delta = datetime.timedelta(seconds=(long(time.time())-long(from_time)))
+                delta = datetime.timedelta(seconds=(int(time.time())-int(from_time)))
                 label.set_text(str(delta))
             v = self.client._remote_version or "unknown"
             if self.client.mmap_enabled:
@@ -301,15 +331,14 @@ class ClientExtrasBase(object):
                 self.server_randr_label.set_text("Yes%s" % size_info)
             else:
                 self.server_randr_label.set_text("No%s" % size_info)
-            if self.client.can_ping:
-                if self.client.server_load:
-                    self.server_load_label.set_text("  ".join([str(x/1000.0) for x in self.client.server_load]))
-                if len(self.client.server_latency)>0:
-                    avg = sum(self.client.server_latency)/len(self.client.server_latency)
-                    self.server_latency_label.set_text("%sms  (%sms)" % (self.client.server_latency[-1], avg))
-                if len(self.client.client_latency)>0:
-                    avg = sum(self.client.client_latency)/len(self.client.client_latency)
-                    self.client_latency_label.set_text("%sms  (%sms)" % (self.client.client_latency[-1], avg))
+            if self.client.server_load:
+                self.server_load_label.set_text("  ".join([str(x/1000.0) for x in self.client.server_load]))
+            if len(self.client.server_latency)>0:
+                avg = int(10*sum(self.client.server_latency)/len(self.client.server_latency))/10
+                self.server_latency_label.set_text("%sms  (%sms)" % (self.client.server_latency[-1], avg))
+            if len(self.client.client_latency)>0:
+                avg = int(10*sum(self.client.client_latency)/len(self.client.client_latency))/10
+                self.client_latency_label.set_text("%sms  (%sms)" % (self.client.client_latency[-1], avg))
             if self.client.server_start_time>0:
                 settimedeltastr(self.session_started_label, self.client.server_start_time)
             else:
@@ -331,17 +360,17 @@ class ClientExtrasBase(object):
                     if v<0:
                         return  "n/a"
                     if v>1000*1000*1000:
-                        return "%sG" % (long(v/1000/1000/100)/10.0)
+                        return "%sG" % (int(v/1000/1000/100)/10.0)
                     elif v>1000*1000:
-                        return "%sM" % (long(v/1000/100)/10.0)
+                        return "%sM" % (int(v/1000/100)/10.0)
                     elif v>1000:
-                        return "%sK" % (long(v/100)/10.0)
+                        return "%sK" % (int(v/100)/10.0)
                     else:
                         return str(v)
                 def fpsstr(v):
                     if v<0:
                         return  "n/a"
-                    return "%s" % (long(v*100)/100.0)
+                    return "%s" % (int(v*100)/100.0)
                 def average(seconds):
                     total = 0
                     total_n = 0
@@ -365,13 +394,13 @@ class ClientExtrasBase(object):
                         return  None
                     avgs = avgs/total_n
                     elapsed = now-startt
-                    return long(total/elapsed), total_n/elapsed, mins, avgs, maxs
+                    return int(total/elapsed), total_n/elapsed, mins, avgs, maxs
                 p20 = average(20)
                 if p20:
                     avg20,fps20,mins,avgs,maxs = p20
                     p1 = average(1)
                     if p1:
-                        avg1,fps1 = average(1)[:2]
+                        avg1,fps1 = p1[:2]
                     else:
                         avg1,fps1 = -1, -1
                     pixels = "%s  (%s)" % (pixelstr(avg1), pixelstr(avg20))
@@ -386,7 +415,8 @@ class ClientExtrasBase(object):
 
         window.set_border_width(15)
         window.add(vbox)
-        window.set_geometry_hints(vbox)
+        if not is_gtk3():
+            window.set_geometry_hints(vbox)
         def window_deleted(*args):
             self.session_info_window = None
         window.connect('delete_event', window_deleted)
@@ -404,9 +434,11 @@ class ClientExtrasBase(object):
             log.error("closing session info", exc_info=True)
 
     def add_close_accel(self, window, callback):
+        if is_gtk3():
+            return      #TODO: implement accel for gtk3
         # key accelerators
         accel_group = gtk.AccelGroup()
-        accel_group.connect_group(ord('w'), gtk.gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, callback)
+        accel_group.connect_group(ord('w'), gdk.CONTROL_MASK, gtk.ACCEL_LOCKED, callback)
         window.add_accel_group(accel_group)
         accel_group = gtk.AccelGroup()
         escape_key, modifier = gtk.accelerator_parse('Escape')
@@ -439,9 +471,9 @@ class ClientExtrasBase(object):
 
     def get_icon_filename(self, icon_name):
         filename = os.path.join(self.get_data_dir(), 'icons', icon_name)
-        log.debug("get_icon_filename(%s)=%s, exists=%s" % (icon_name, filename, os.path.exists(filename)))
         if os.path.exists(filename):
             return  filename
+        log.error("get_icon_filename(%s) %s does not exists!" % (icon_name, filename))
         return  None
 
     def get_license_text(self):
@@ -450,10 +482,13 @@ class ClientExtrasBase(object):
         filename = os.path.join(self.get_data_dir(), 'COPYING')
         if os.path.exists(filename):
             try:
-                file = open(filename, mode='rb')
-                return file.read()
+                if sys.version < '3':
+                    license_file = open(filename, mode='rb')
+                else:
+                    license_file = open(filename, mode='r', encoding='ascii')
+                return license_file.read()
             finally:
-                file.close()
+                license_file.close()
         if not self.license_text:
             self.license_text = "GPL version 2"
         return self.license_text
@@ -461,9 +496,15 @@ class ClientExtrasBase(object):
     def get_pixbuf(self, icon_name):
         try:
             icon_filename = self.get_icon_filename(icon_name)
-            return  gtk.gdk.pixbuf_new_from_file(icon_filename)
+            if icon_filename:
+                if is_gtk3():
+                    from gi.repository.GdkPixbuf import Pixbuf    #@UnresolvedImport
+                    return Pixbuf.new_from_file(icon_filename)
+                else:
+                    return  gdk.pixbuf_new_from_file(icon_filename)
         except:
-            return  None
+            log.error("get_image(%s)", icon_name, exc_info=True)
+        return  None
 
     def get_image(self, icon_name, size=None):
         try:
@@ -471,9 +512,17 @@ class ClientExtrasBase(object):
             if not pixbuf:
                 return  None
             if size:
-                pixbuf = pixbuf.scale_simple(size, size, gtk.gdk.INTERP_BILINEAR)
+                if is_gtk3():
+                    from gi.repository.GdkPixbuf import InterpType  #@UnresolvedImport
+                    interp = InterpType.BILINEAR
+                else:
+                    interp = gdk.INTERP_BILINEAR
+                pixbuf = pixbuf.scale_simple(size, size, interp)
+            if is_gtk3():
+                return  gtk.Image.new_from_pixbuf(pixbuf)
             return  gtk.image_new_from_pixbuf(pixbuf)
         except:
+            log.error("get_image(%s, %s)", icon_name, size, exc_info=True)
             return  None
 
 
@@ -489,10 +538,10 @@ class ClientExtrasBase(object):
             #override gtk defaults: we *want* icons:
             settings = menu_item.get_settings()
             settings.set_property('gtk-menu-images', True)
-            if hasattr(settings, "set_always_show_image"):
-                settings.set_always_show_image(True)
+            if hasattr(menu_item, "set_always_show_image"):
+                menu_item.set_always_show_image(True)
         if tooltip:
-            menu_item.set_tooltip_text(tooltip)
+            set_tooltip_text(menu_item, tooltip)
         if cb:
             menu_item.connect('activate', cb)
         menu_item.show()
@@ -500,7 +549,7 @@ class ClientExtrasBase(object):
 
     def checkitem(self, title, cb=None):
         """ Utility method for easily creating a CheckMenuItem """
-        check_item = gtk.CheckMenuItem(title)
+        check_item = CheckMenuItem(title)
         if cb:
             check_item.connect("toggled", cb)
         check_item.show()
@@ -523,7 +572,10 @@ class ClientExtrasBase(object):
 
     def show_menu(self, button, time):
         self.close_menu()
-        self.menu.popup(None, None, None, button, time, None)
+        if is_gtk3():
+            self.menu.popup(None, None, None, None, button, time)
+        else:
+            self.menu.popup(None, None, None, button, time, None)
         self.menu_shown = True
 
     def make_aboutmenuitem(self):
@@ -535,22 +587,38 @@ class ClientExtrasBase(object):
     def make_bellmenuitem(self):
         def bell_toggled(*args):
             self.client.bell_enabled = self.bell_menuitem.get_active()
+            self.client.send_bell_enabled()
             log.debug("bell_toggled(%s) bell_enabled=%s", args, self.client.bell_enabled)
         self.bell_menuitem = self.checkitem("Bell", bell_toggled)
-        def set_bellmenuitem(*args):
-            self.bell_menuitem.set_active(self.client.bell_enabled)
-            self.bell_menuitem.set_sensitive(self.client.server_capabilities.get("bell", False))
-        self.client.connect("handshake-complete", set_bellmenuitem)
+        set_tooltip_text(self.bell_menuitem, "Forward system bell")
+        self.bell_menuitem.set_active(self.client.bell_enabled)
         return  self.bell_menuitem
 
+    def make_cursorsmenuitem(self):
+        def cursors_toggled(*args):
+            self.client.cursors_enabled = self.cursors_menuitem.get_active()
+            self.client.send_cursors_enabled()
+            log.debug("cursors_toggled(%s) cursors_enabled=%s", args, self.client.cursors_enabled)
+        self.cursors_menuitem = self.checkitem("Cursors", cursors_toggled)
+        set_tooltip_text(self.cursors_menuitem, "Forward custom mouse cursors")
+        self.cursors_menuitem.set_active(self.client.cursors_enabled)
+        return  self.cursors_menuitem
+
     def make_notificationsmenuitem(self):
         def notifications_toggled(*args):
             self.client.notifications_enabled = self.notifications_menuitem.get_active()
+            self.client.send_notify_enabled()
             log.debug("notifications_toggled(%s) notifications_enabled=%s", args, self.client.notifications_enabled)
         self.notifications_menuitem = self.checkitem("Notifications", notifications_toggled)
         def set_notifications_menuitem(*args):
             self.notifications_menuitem.set_active(self.client.notifications_enabled)
-            self.notifications_menuitem.set_sensitive(self.client.server_capabilities.get("notifications", False))
+            can_notify = self.client.server_capabilities.get("notifications", False)
+            self.notifications_menuitem.set_sensitive(can_notify)
+            if can_notify:
+                set_tooltip_text(self.notifications_menuitem, "Forward system notifications")
+            else:
+                set_tooltip_text(self.notifications_menuitem, "Cannot forward system notifications: disabled by server")
+            set_tooltip_text(self.cursors_menuitem, "Forward custom mouse cursors")
         self.client.connect("handshake-complete", set_notifications_menuitem)
         return self.notifications_menuitem
 
@@ -562,7 +630,12 @@ class ClientExtrasBase(object):
         self.clipboard_menuitem = self.checkitem("Clipboard", clipboard_toggled)
         def set_clipboard_menuitem(*args):
             self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
-            self.clipboard_menuitem.set_sensitive(self.client.server_capabilities.get("clipboard", True))
+            can_clip = self.client.server_capabilities.get("clipboard", False)
+            self.clipboard_menuitem.set_sensitive(can_clip)
+            if can_clip:
+                set_tooltip_text(self.clipboard_menuitem, "Enable clipboard synchronization")
+            else:
+                set_tooltip_text(self.clipboard_menuitem, "Clipboard synchronization cannot be enabled: disabled by server")
         self.client.connect("handshake-complete", set_clipboard_menuitem)
         return self.clipboard_menuitem
 
@@ -575,11 +648,10 @@ class ClientExtrasBase(object):
             if self.client.mmap_enabled:
                 #mmap disables encoding and uses raw rgb24
                 encodings.set_label("Encoding")
-                encodings.set_tooltip_text("memory mapped transfers are in use so picture encoding is disabled")
+                set_tooltip_text(encodings, "memory mapped transfers are in use so picture encoding is disabled")
                 encodings.set_sensitive(False)
             for encoding in ENCODINGS:
-                encoding_item = gtk.CheckMenuItem(encoding)
-                encoding_item.get_label()
+                encoding_item = CheckMenuItem(encoding)
                 def encoding_changed(item):
                     item = ensure_item_selected(self.encodings_submenu, item)
                     enc = item.get_label()
@@ -629,7 +701,7 @@ class ClientExtrasBase(object):
                 self.layout_submenu.append(kbitem("%s - %s" % (layout, v), layout, v))
         else:
             #show all options to choose from:
-            sorted_keys = X11_LAYOUTS.keys()
+            sorted_keys = list(X11_LAYOUTS.keys())
             sorted_keys.sort(key=keysort)
             for key in sorted_keys:
                 country,language = key
@@ -649,9 +721,9 @@ class ClientExtrasBase(object):
                     #no variants:
                     self.layout_submenu.append(kbitem(name, layout, None))
         def set_selected_layout(*args):
-            if self.client._raw_keycodes_full:
-                #server supports raw keycode and we do too
-                #so no need to let the user select the keymap
+            if self.client.xkbmap_layout or self.client.xkbmap_print or self.client.xkbmap_query:
+                #we have detected a layout
+                #so no need to let the user override it
                 keyboard.hide()
                 return
             layout = self.client.xkbmap_layout
@@ -683,7 +755,7 @@ class ClientExtrasBase(object):
                 log.debug("setting jpeg quality to %s", q)
                 self.client.send_jpeg_quality(q)
         for q in jpeg_options:
-            qi = gtk.CheckMenuItem("%s%%" % q)
+            qi = CheckMenuItem("%s%%" % q)
             qi.set_draw_as_radio(True)
             qi.set_active(q==self.client.jpegquality)
             qi.connect('activate', set_jpeg_quality)
@@ -711,7 +783,7 @@ class ClientExtrasBase(object):
                 self.client.compression_level = c
                 self.client.send_deflate_level()
         for i in range(0, 10):
-            c = gtk.CheckMenuItem(str(compression_options.get(i, i)))
+            c = CheckMenuItem(str(compression_options.get(i, i)))
             c.set_draw_as_radio(True)
             c.set_active(i==self.client.compression_level)
             c.connect('activate', set_compression)
@@ -756,6 +828,7 @@ class ClientExtrasBase(object):
         menu.append(self.make_sessioninfomenuitem())
         menu.append(gtk.SeparatorMenuItem())
         menu.append(self.make_bellmenuitem())
+        menu.append(self.make_cursorsmenuitem())
         menu.append(self.make_notificationsmenuitem())
         if not self.client.readonly:
             menu.append(self.make_clipboardmenuitem())
