@@ -8,11 +8,10 @@
 
 import sys
 import os
-
-import pygtk
-pygtk.require("2.0")
-import gtk
-_display = gtk.gdk.get_display()
+from wimpiggy.gobject_compat import import_gtk, import_gdk
+gtk = import_gtk()
+gdk = import_gdk()
+_display = gdk.get_display()
 assert _display, "cannot open the display with GTK, is DISPLAY set?"
 
 from xpra.platform.client_extras_base import ClientExtrasBase
@@ -80,16 +79,22 @@ class ClientExtras(ClientExtrasBase):
             self.tray_widget.connect('activate', self.activate_menu)
             filename = self.get_tray_icon_filename(tray_icon_filename)
             if filename:
-                pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
-                self.tray_widget.set_from_pixbuf(pixbuf)
+                if hasattr(self.tray_widget, "set_from_file"):
+                    self.tray_widget.set_from_file(filename)
+                else:
+                    pixbuf = gdk.pixbuf_new_from_file(filename)
+                    self.tray_widget.set_from_pixbuf(pixbuf)
             def hide_tray(*args):
                 self.tray_widget.set_visible(False)
             self.hide_tray = hide_tray
             def show_tray(*args):
                 log.debug("showing tray")
                 #session_name will get set during handshake
-                self.tray_widget.set_tooltip(self.client.session_name)
                 self.tray_widget.set_visible(True)
+                if hasattr(self.tray_widget, "set_tooltip_text"):
+                    self.tray_widget.set_tooltip_text(self.client.session_name)
+                else:
+                    self.tray_widget.set_tooltip(self.client.session_name)
             self.client.connect("handshake-complete", show_tray)
             return True
         except Exception, e:
@@ -171,7 +176,8 @@ class ClientExtras(ClientExtrasBase):
             self.has_dbusnotify = True
             log("using dbusnotify: %s(%s)", type(self.dbusnotify), FD_NOTIFICATIONS)
         except Exception, e:
-            log.error("cannot import dbus.glib notification wrapper: %s", e)
+            log("cannot import dbus.glib notification wrapper: %s", e)
+            log.error("failed to locate the dbus notification service")
         return self.has_dbusnotify
 
     def setup_pynotify(self):
@@ -229,16 +235,20 @@ class ClientExtras(ClientExtrasBase):
 
     def system_bell(self, window, device, percent, pitch, duration, bell_class, bell_id, bell_name):
         if not self.has_x11_bell:
-            import gtk.gdk
-            gtk.gdk.beep()
+            gdk.beep()
             return
-        from wimpiggy.lowlevel.bindings import device_bell      #@UnresolvedImport
-        device_bell(window, device, bell_class, bell_id, percent, bell_name)
+        from wimpiggy.error import trap, XError
+        try:
+            from wimpiggy.lowlevel.bindings import device_bell      #@UnresolvedImport
+            trap.call_unsynced(device_bell, window, device, bell_class, bell_id, percent, bell_name)
+        except XError, e:
+            log.error("error using device_bell: %s, will fallback to gdk beep from now on", e)
+            self.has_x11_bell = False
 
     def can_notify(self):
         return  self.has_dbusnotify or self.has_pynotify
 
-    def show_notify(self, dbus_id, id, app_name, replaces_id, app_icon, summary, body, expire_timeout):
+    def show_notify(self, dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, expire_timeout, may_retry=True):
         if self.dbus_id==dbus_id:
             log.error("remote dbus instance is the same as our local one, "
                       "cannot forward notification to ourself as this would create a loop")
@@ -247,8 +257,28 @@ class ClientExtras(ClientExtrasBase):
             def cbReply(*args):
                 log("notification reply: %s", args)
                 return False
-            def cbError(*args):
-                log.error("notification error: %s", args)
+            def cbError(dbus_error, *args):
+                try:
+                    import dbus.exceptions
+                    if type(dbus_error)==dbus.exceptions.DBusException:
+                        message = dbus_error.get_dbus_message()
+                        dbus_error_name = dbus_error.get_dbus_name()
+                        if dbus_error_name!="org.freedesktop.DBus.Error.ServiceUnknown":
+                            log.error("unhandled dbus exception: %s, %s", message, dbus_error_name)
+                            return False
+
+                        if not may_retry:
+                            log.error("cannot send notification via dbus, please check that you notification service is operating properly")
+                            return False
+
+                        log.info("trying to re-connect to the notification service")
+                        #try to connect to the notification again (just once):
+                        if self.setup_dbusnotify():
+                            self.show_notify(dbus_id, nid, app_name, replaces_nid, app_icon, summary, body, expire_timeout, may_retry=False)
+                        return False
+                except:
+                    pass
+                log.error("notification error: %s", dbus_error)
                 return False
             try:
                 self.dbusnotify.Notify("Xpra", 0, app_icon, summary, body, [], [], expire_timeout,
@@ -268,7 +298,7 @@ class ClientExtras(ClientExtrasBase):
         else:
             log.error("notification cannot be displayed, no backend support!")
 
-    def close_notify(self, id):
+    def close_notify(self, nid):
         pass
 
     def exec_get_keyboard_data(self, cmd):
@@ -278,7 +308,7 @@ class ClientExtras(ClientExtrasBase):
             process = subprocess.Popen(cmd, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False)
             (out,_) = process.communicate(None)
             if process.returncode==0:
-                return out
+                return out.decode('utf-8')
             log.error("'%s' failed with exit code %s", cmd, process.returncode)
         except Exception, e:
             log.error("error running '%s': %s", cmd, e)
@@ -292,14 +322,12 @@ class ClientExtras(ClientExtrasBase):
                 #ie: {"shift" : ["Shift_L", "Shift_R"], "mod1" : "Meta_L", ...]}
                 log.debug("modifier mappings=%s", mod_mappings)
                 meanings = {}
-                clear = []
-                add = []
                 for modifier,keynames in mod_mappings.items():
-                    clear.append("clear %s" % modifier)
-                    add.append("add %s = %s" % (modifier, " ".join(keynames)))
                     for keyname in keynames:
                         meanings[keyname] = modifier
-                return  clear, add, meanings, [], []
+                return  meanings, [], []
+        except ImportError, e:
+            log.error("failed to use native get_modifier_mappings: %s", e)
         except Exception, e:
             log.error("failed to use native get_modifier_mappings: %s", e, exc_info=True)
         return self.modifiers_fallback()
@@ -307,25 +335,22 @@ class ClientExtras(ClientExtrasBase):
     def modifiers_fallback(self):
         xmodmap_pm = self.exec_get_keyboard_data(["xmodmap", "-pm"])
         if not xmodmap_pm:
+            log.warn("bindings are not available and 'xmodmap -pm' also failed, expect keyboard mapping problems")
             return ClientExtrasBase.get_keymap_modifiers(self)
         #parse it so we can feed it back to xmodmap (ala "xmodmap -pke")
-        clear = []
-        add = []
         meanings = {}
         for line in xmodmap_pm.splitlines()[1:]:
             if not line:
                 continue
             parts = line.split()
             #ie: ['shift', 'Shift_L', '(0x32),', 'Shift_R', '(0x3e)']
-            clear.append("clear %s" % parts[0])
             if len(parts)>1:
                 nohex = [x for x in parts[1:] if not x.startswith("(")]
-                add.append("add %s = %s" % (parts[0], " ".join(set(nohex))))
                 for x in nohex:
                     #ie: meanings['Shift_L']=shift
                     meanings[x] = parts[0]
-        log.debug("get_keymap_modifiers parsed: clear=%s, add=%s, meanings=%s", clear, add, meanings)
-        return  clear, add, meanings, [], []
+        log.debug("get_keymap_modifiers parsed: meanings=%s", meanings)
+        return  meanings, [], []
 
     def get_keymap_spec(self):
         xkbmap_print = self.exec_get_keyboard_data(["setxkbmap", "-print"])
@@ -335,8 +360,7 @@ class ClientExtras(ClientExtrasBase):
         if xkbmap_query is None and xkbmap_print is not None:
             log.error("the server will try to guess your keyboard mapping, which works reasonably well in most cases");
             log.error("however, upgrading 'setxkbmap' to a version that supports the '-query' parameter is preferred");
-        xmodmap_data = self.exec_get_keyboard_data(["xmodmap", "-pke"])
-        return xkbmap_print, xkbmap_query, xmodmap_data
+        return xkbmap_print, xkbmap_query
 
     def get_keyboard_repeat(self):
         try:
